﻿using KTour.Common;
using KTour.Views;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace KTour
{
    sealed partial class App : Application
    {
        /// <summary>
        /// 엑티베이트가 되면 true
        /// </summary>
        private static bool IsActivate;

        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
        }

        /// <summary>
        /// 앱 시작
        /// </summary>
        /// <param name="args">Splash screen</param>
        /// <param name="queryText">QueryText</param>
        /// <returns></returns>
        private static void InitApp(object args, string queryText = null)
        {
            ExtendedSplash extendedSplash = null;
            switch (args.GetType().Name)
            {
                case "LaunchActivatedEventArgs":
                    var lancher = args as LaunchActivatedEventArgs;
                    bool loadState = (lancher.PreviousExecutionState == ApplicationExecutionState.Terminated);
                    extendedSplash = new ExtendedSplash(lancher.SplashScreen, loadState);
                    extendedSplash.ViewModel.NavigateViewName = "MainView";
                    break;
                case "SearchActivatedEventArgs":
                    var searcher = args as SearchActivatedEventArgs;
                    bool loadStateSearch = (searcher.PreviousExecutionState == ApplicationExecutionState.Terminated);
                    extendedSplash = new ExtendedSplash(searcher.SplashScreen, loadStateSearch);
                    extendedSplash.ViewModel.NavigateViewName = "SearchResultView";
                    extendedSplash.ViewModel.QueryString = searcher.QueryText;
                    break;
            }
            Window.Current.Content = extendedSplash;
        }

        protected override void OnLaunched(LaunchActivatedEventArgs args)
        {
            if (args.PreviousExecutionState != ApplicationExecutionState.Running)
            {
                InitApp(args as object, null);
            }
            Window.Current.Activate();
        }

        private async void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            await SuspensionManager.SaveAsync();
            deferral.Complete();
        }

        /// <summary>
        /// Invoked when the application is activated to display search results.
        /// </summary>
        /// <param name="args">Details about the activation request.</param>
        protected override void OnSearchActivated(Windows.ApplicationModel.Activation.SearchActivatedEventArgs args)
        {
            var previousContent = Window.Current.Content;
            var frame = previousContent as Frame;

            if (frame != null)
            {
                //앱이 실행된 상태라면 그냥 네비게이션 해줌
                if (frame.SourcePageType.Name != "SearchResultView")
                {
                    // 앱이 실행 중이고 검색화면이 아니면 검색화면으로 네비게이션
                    frame.Navigate(typeof(KTour.Views.SearchResultView), args.QueryText);
                }
                else
                {
                    // 앱이 실행 중이고 현재 화면이 검색화면임
                    var locator = App.Current.Resources["Locator"] as KTour.ViewModels.ViewModelLocator;
                    locator.SearchResultVM.AddFilter(args.QueryText);
                }
            }
            else
            {
                InitApp(args, args.QueryText);
            }

            if (IsActivate == false)
            {
                Window.Current.Activate();
                IsActivate = true;
            }
        }
    }
}
