﻿using System;
using System.Xml.Linq;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI;
using Windows.UI.Xaml.Shapes;
using Windows.UI.Xaml.Markup;
using Windows.UI.Xaml.Media;
//완전 중요;;;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.ApplicationModel.Resources.Core;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.Foundation;
using System.Threading;

namespace KTour.Common
{
    public static class Functions
    {
        static MessageDialog msg;
        static bool isShow;

        public static string ApiKey 
        {
            get 
            {
                //https://www.data.go.kr/main.jsp
                var key = Uri.EscapeDataString("국가공유자원포털에서 받은 API키를 입력하시기 바랍니다.");
                return key; 
            }
        }

        public const string MobileOS = "WIN";
        public const string MobileApp = "KoreaTourGuide";
        
        /// <summary>
        /// 숫자로
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static int GetInt(object element)
        {
            int returnValue = -1;

            if (element != null)
            {
                switch (element.GetType().Name)
                { 
                    case "XElement":
                        XElement x = element as XElement;
                        returnValue = Convert.ToInt32(string.IsNullOrEmpty(x.Value) ? "0" : x.Value);
                        break;
                }
            }
            return returnValue;
        }

        /// <summary>
        /// 문자로
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static string GetString(object element)
        {
            string returnValue = string.Empty;

            if (element != null)
            {
                switch (element.GetType().Name)
                {
                    case "XElement":
                        XElement x = element as XElement;
                        returnValue = x.Value.Trim();
                        break;
                }
            }
            return returnValue;
        }

        /// <summary>
        /// 날짜로
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static DateTime GetDateTime(object element)
        {
            DateTime returnValue = DateTime.MinValue;

            if (element != null)
            {
                switch (element.GetType().Name)
                {
                    case "XElement":
                        XElement x = element as XElement;
                        returnValue = DateTime.Parse(x.Value);
                        break;
                }
            }
            return returnValue;
        }

        /// <summary>
        /// 더블형
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static double GetDouble(object element)
        {
            double returnValue = -1;

            if (element != null)
            {
                switch (element.GetType().Name)
                {
                    case "XElement":
                        XElement x = element as XElement;
                        returnValue = Convert.ToDouble(x.Value);
                        break;
                }
            }
            return returnValue;
        }

        /// <summary>
        /// 싱글형
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static Single GetSingle(object element)
        {
            Single returnValue = -1;

            if (element != null)
            {
                switch (element.GetType().Name)
                {
                    case "XElement":
                        XElement x = element as XElement;
                        returnValue = Convert.ToSingle(x.Value);
                        break;
                }
            }
            return returnValue;
        }

        /// <summary>
        /// 메시지 박스 출력
        /// </summary>
        /// <param name="message"></param>
        public static async void MsgBox(string message)
        {
            if (msg == null)
            {
                msg = new MessageDialog(message, GetResString("Message_Notification"));
            }
            if (isShow == false)
            {
                isShow = true;
                var result = await msg.ShowAsync();
                isShow = false;
            }
        }

        /// <summary>
        /// 확인 박스 출력
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public static async System.Threading.Tasks.Task<bool> Confirm(string message)
        {
            MessageDialog msg = new MessageDialog(message, GetResString("Message_Confirm"));
            msg.Commands.Add(new UICommand("OK", _ => { }, "0"));
            msg.Commands.Add(new UICommand("Cancel", _ => { }, "1"));
            msg.DefaultCommandIndex = 1;
            var result = await msg.ShowAsync();
            if (result.Id.ToString() == "0" )
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// Creates a FileOpenPicker for the specified extensions. 
        /// </summary>
        /// <param name="extensions">File extensions to pick.</param>
        /// <returns>FileOpenPicker instance.</returns>
        public static FileOpenPicker CreateFilePicker(string[] extensions, PickerLocationId locationId)
        {
            FileOpenPicker picker = new FileOpenPicker();
            picker.SuggestedStartLocation = locationId;
            foreach (string extension in extensions)
            {
                picker.FileTypeFilter.Add(extension);
            }
            return picker;
        }

        /// <summary>
        /// 색상을 컬러로 바꿔주는 함수 #FFRRGGBB
        /// </summary>
        /// <param name="rgbColor"></param>
        /// <returns></returns>
        public static Color FromStringColor(string rgbColor)
        {
            Color c = new Color();
            //byte a = 255; // or whatever...
            byte a = (byte)(Convert.ToUInt32(rgbColor.Substring(1, 2), 16));
            byte r = (byte)(Convert.ToUInt32(rgbColor.Substring(3, 2), 16));
            byte g = (byte)(Convert.ToUInt32(rgbColor.Substring(5, 2), 16));
            byte b = (byte)(Convert.ToUInt32(rgbColor.Substring(7, 2), 16));
            c = Color.FromArgb(a, r, g, b);
            return c;
        }

        /// <summary>
        /// 문자열 컬러 리턴
        /// </summary>
        /// <param name="colorName"></param>
        /// <returns></returns>
        public static Color FromKnownColor(string colorName)
        {
            Line lne = (Line)XamlReader.Load("<Line xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\" xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\" Fill=\"" + colorName + "\" />");
            return (Color)lne.Fill.GetValue(SolidColorBrush.ColorProperty);
        }

        /// <summary>
        /// 파일 실행
        /// </summary>
        /// <param name="filePath"></param>
        public static async void LaunchFile(string filePath)
        {
            bool success = false;

            var file = await Windows.Storage.StorageFile.GetFileFromPathAsync(filePath);
            if (file != null)
            {
                success = await Windows.System.Launcher.LaunchFileAsync(file);
            }
        }

        /// <summary>
        /// 페이지 네비게이션
        /// </summary>
        /// <param name="navigationType"></param>
        /// <param name="navigationParameter"></param>
        /// <returns></returns>
        public static bool NavigationPage(Type navigationType, string navigationParameter)
        {
            var mainFrame = Windows.UI.Xaml.Window.Current.Content as Windows.UI.Xaml.Controls.Frame;
            var result = mainFrame.Navigate(navigationType, navigationParameter);
            return result;
        }

        //간단한 데이터 저장을 위한 함수
        //http://msdn.microsoft.com/en-us/library/windows/apps/windows.storage.applicationdata.roamingsettings.aspx
        public static void SaveRoamingSetting(string key, object value)
        {
            Windows.Storage.ApplicationData.Current.RoamingSettings.Values[key] = value;
        }

        //간단한 데이터 로드를 위한 함수
        public static object LoadRoamingSetting(string key)
        {
            return Windows.Storage.ApplicationData.Current.RoamingSettings.Values[key];
        }

        /// <summary>
        /// 언어 리소스 리턴 함수
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static string GetResString(string key)
        {
            string returnValue = string.Empty;
            var resources = ResourceManager.Current.MainResourceMap.GetSubtree("Resources");
            var result = resources.GetValue(key);
            if (result != null) returnValue = result.ValueAsString;
            return returnValue;
        }

        /// <summary>
        /// 풀사이즈 팝업 오픈
        /// </summary>
        /// <param name="contentType">UserControl</param>
        public static Popup OpenPopupFullSize(Type userControl)
        {
            var bounds = Windows.UI.Xaml.Window.Current.CoreWindow.Bounds;
            var popup = new Popup();
            var content = System.Activator.CreateInstance(userControl) as Windows.UI.Xaml.Controls.UserControl;
            if (content == null) return null;
            content.Width = bounds.Width;
            content.Height = bounds.Height;
            popup.Child = content;
            popup.IsOpen = true;
            return popup;
        }

        /// <summary>
        /// 작은 팝업 오픈
        /// </summary>
        /// <param name="contentType">유저컨트롤</param>
        /// <param name="element">위치지정에 사용되는 엘리먼트</param>
        /// <returns></returns>
        public static Popup OpenPopupElement(Type contentType, FrameworkElement element)
        {
            var popup = new Popup();
            var bounds = Windows.UI.Xaml.Window.Current.CoreWindow.Bounds;
            var content = System.Activator.CreateInstance(contentType) as UserControl;
            if (content == null) return null;
            //엘리먼트위치
            var rect = GetElementRect(element);
            popup.VerticalOffset = rect.Top + rect.Height + 4;
            //popup.HorizontalOffset = rect.Left + content.Width > bounds.Width ? bounds.Width - (content.Width + 24) : rect.Left;
            content.Width = bounds.Width;
            popup.Child = content;
            popup.IsLightDismissEnabled = true;
            popup.IsOpen = true;
            return popup;
        }

        /// <summary>
        /// 엘리먼트의 위치와 크기 반환
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static Rect GetElementRect(FrameworkElement element)
        {
            GeneralTransform buttonTransform = element.TransformToVisual(null);
            Point point = buttonTransform.TransformPoint(new Point());
            return new Rect(point, new Size(element.ActualWidth, element.ActualHeight));
        }

        /// <summary>
        /// 스래드 통합 - 다른 스래드에서 작업한 내용을 현재 스래드로 합쳐서 작업한다.
        /// </summary>
        /// <param name="context"></param>
        /// <param name="callback"></param>
        /// <param name="state"></param>
        public static void InvokeIfRequiredAsync(SynchronizationContext context, SendOrPostCallback callback, object state)
        {
            context.Post(callback, state); //모든 프레임웍에서 동작할려면 이거 써야한다는데..
        }

        /// <summary>
        /// 모델 뷰모델 모두 사용 - 기본 컨텍스트
        /// </summary>
        public static System.Threading.SynchronizationContext BaseContext { get; set; }

    }
}
