﻿using KTour.Common;
using KTour.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace KTour.OpenAPI
{
    public abstract class BaseService<T> : KTour.Common.BindableBase
    {
        /// <summary>
        /// 서비스 Uri
        /// </summary>
        public string ServiceUri { get; set; }

        private ObservableCollection<T> resultCollection;
        /// <summary>
        /// 결과 컬렉션
        /// </summary>
        public ObservableCollection<T> ResultCollection 
        {
            get { return resultCollection; }
            set 
            {
                resultCollection = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        /// 결과코드
        /// </summary>
        public string ResultCode { get; set; }

        /// <summary>
        /// 결과메시지
        /// </summary>
        public string ResultMsg { get; set; }

        /// <summary>
        /// 반환갯수
        /// </summary>
        public int NumOfRows { get; set; }

        private int pageNo;
        /// <summary>
        /// 페이지 번호
        /// </summary>
        public int PageNo
        {
            get { return pageNo; }
            set 
            {
                pageNo = value;
                OnPropertyChanged();
            }
        }

        private int totalCount;
        /// <summary>
        /// 전체 카운트
        /// </summary>
        public int TotalCount
        {
            get { return totalCount; }
            set 
            {
                totalCount = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        /// 토탈 페이지 = totalCount / NumOfRows + 1
        /// </summary>
        public int TotalPage { get; set; }

        /// <summary>
        /// 정렬 구분 (A=제목순, B=조회순, C=수정일순, D=생성일순)
        /// </summary>
        public string Arrange { get; set; }

        /// <summary>
        /// 생성자
        /// </summary>
        public BaseService()
        {
            ResultCollection = new ObservableCollection<T>();
            ServiceUri = "";
            NumOfRows = 30;
            Arrange = "C";
            PageNo = 1;

            this.PropertyChanged += 
                (s, e) => 
                {
                    switch (e.PropertyName)
                    { 
                        case "TotalCount":
                            if (TotalCount > 0)
                            {
                                TotalPage = TotalCount / NumOfRows;
                                if ((TotalCount % NumOfRows) > 0)
                                    TotalPage++;
                            }
                            else
                            {
                                TotalPage = 0;
                            }
                            break;
                    }
                };
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="para">파라메터</param>
        /// <returns></returns>
        public abstract Task Send(string para);
    }
}
