﻿using KTour.Common;
using KTour.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace KTour.OpenAPI
{
    public class FestivalService : BaseService<FestivalModel>
    {
        /// <summary>
        /// 검색 - 행사시작일자
        /// </summary>
        public string EventStartDate { get; set; }

        /// <summary>
        /// 검색 - 행사종료일자
        /// </summary>
        public string EventEndDate { get; set; }

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public FestivalService()
        {
            ServiceUri = "http://api.visitkorea.or.kr/openapi/service/rest/KorService/searchFestival?ServiceKey={0}&eventStartDate={1}&eventEndDate={2}&arrange={3}&listYN=Y&pageNo={4}&numOfRows={5}&MobileOS={6}&MobileApp={7}";
            EventStartDate = "";
            EventEndDate = "";
        }

        /// <summary>
        /// 서비스 조회
        /// </summary>
        /// <param name="para">시작일자|종료일자</param>
        /// <returns></returns>
        public override async Task Send(string para)
        {
            using (HttpClient hc = new HttpClient())
            {
                try
                {
                    if (para != null)
                    {
                        var paras = para.Split('|');
                        EventStartDate = paras.Count() >= 1 ? paras[0] : "";
                        EventEndDate = paras.Count() >= 2 ? paras[1] : "";
                    }

                    var uri = string.Format(ServiceUri, Functions.ApiKey, EventStartDate, EventEndDate, Arrange, PageNo, NumOfRows, Functions.MobileOS, Functions.MobileApp);
                    var result = await hc.GetStringAsync(uri);
                    if (!string.IsNullOrEmpty(result))
                    {
                        XElement xml = XElement.Parse(result);

                        var header = xml.Element("header");
                        ResultCode = Functions.GetInt(header.Element("resultCode")).ToString();
                        ResultMsg = Functions.GetString(header.Element("resultMsg"));
                        var body = xml.Element("body");
                        PageNo = Functions.GetInt(body.Element("pageNo"));
                        TotalCount = Functions.GetInt(body.Element("totalCount"));

                        switch (ResultCode)
                        {
                            case "0":   //정상
                                var rows = from kkk in xml.Descendants("item")
                                           let mtime = Convert.ToInt32(Functions.GetString(kkk.Element("modifiedtime")).Substring(0,8)).ToString("####-##-##")
                                           select new FestivalModel 
                                            {
                                                Addr1 = Functions.GetString(kkk.Element("addr1")),
                                                Addr2 = Functions.GetString(kkk.Element("addr2")),
                                                AreaCode = Functions.GetString(kkk.Element("areacode")),
                                                Cat1 = Functions.GetString(kkk.Element("cat1")),
                                                Cat2 = Functions.GetString(kkk.Element("cat2")),
                                                Cat3 = Functions.GetString(kkk.Element("cat3")),
                                                ContentId = Functions.GetString(kkk.Element("contentid")),
                                                ContentTypeId = Functions.GetString(kkk.Element("contenttypeid")),
                                                CreateTime = Functions.GetString(kkk.Element("createdtime")),
                                                FirstImage = new Uri(Functions.GetString(kkk.Element("firstimage"))),
                                                FirstImage2 = new Uri(Functions.GetString(kkk.Element("firstimage2"))),
                                                ModifiedTime = mtime,
                                                ReadCount = Functions.GetString(kkk.Element("readcount")),
                                                SiGunGuCode = Functions.GetString(kkk.Element("sigungucode")),
                                                Title = Functions.GetString(kkk.Element("title")),
                                                MapX = Functions.GetString(kkk.Element("mapx")),
                                                MapY = Functions.GetString(kkk.Element("mapy")),
                                                MLevel = Functions.GetString(kkk.Element("mlevel")),
                                                BookTour = Functions.GetString(kkk.Element("booktour")),
                                                Tel = Functions.GetString(kkk.Element("tel")),
                                                EventStartDate = Functions.GetString(kkk.Element("eventstartdate")),
                                                EventEndDate = Functions.GetString(kkk.Element("eventenddate"))
                                            };
                                ResultCollection = new System.Collections.ObjectModel.ObservableCollection<FestivalModel>(rows);
                                break;
                            case "22":  //서비스 요청제한횟수 초과에러
                                break;
                            default:
                                Functions.MsgBox("조회 중 오류가 발생했습니다.\n" + ResultMsg);
                                ResultCollection.Clear();
                                break;
                        }

                    }
                }
                catch (Exception ex)
                {
                    Functions.MsgBox("오류" + ex.Message);
                }
            }
        }
    }
}
