﻿using KTour.Common;
using KTour.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace KTour.OpenAPI
{
    public class SearchKeywordService : BaseService<ResultBaseModel>
    {
        public bool IsBusy { get; set; }

        public string Keyword { get; set; }

        public string ContentTypeId { get; set; }

        public SearchKeywordService()
        {
            //문서에 나온 순서대로 작업
            ServiceUri = "http://api.visitkorea.or.kr/openapi/service/rest/KorService/searchKeyword?ServiceKey={0}&numOfRows={1}&pageNo={2}&Arrange={3}&listYN={4}&MobileOS={5}&MobileApp={6}&contentTypeId={7}&keyword={8}";
            NumOfRows = 30;
            Arrange = "A";
            IsBusy = false;
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="para">키워드|컨텐츠타입Id</param>
        /// <returns></returns>
        public override async Task Send(string para)
        {
            IsBusy = true;

            using (HttpClient hc = new HttpClient())
            {
                try
                {
                    if (para != null)
                    {
                        var paras = para.Split('|');
                        Keyword = paras.Count() >= 1 ? paras[0] : "";
                        ContentTypeId = paras.Count() >= 2 ? paras[1] : "";
                        PageNo = 1;
                    }
                    else
                    {
                        //파라메터가 널이면 다음 페이지 호출
                        PageNo++;
                    }

                    var uri = string.Format(ServiceUri, Functions.ApiKey, NumOfRows, PageNo, Arrange, "Y", Functions.MobileOS, Functions.MobileApp, ContentTypeId, Uri.EscapeUriString(Keyword) );
                    var result = await hc.GetStringAsync(uri);
                    if (!string.IsNullOrEmpty(result))
                    {
                        XElement xml = XElement.Parse(result);

                        var header = xml.Element("header");
                        if (header == null)
                        {
                            Functions.MsgBox("서비스 오류 입니다.");
                            return;
                        }
                        ResultCode = Functions.GetInt(header.Element("resultCode")).ToString();
                        ResultMsg = Functions.GetString(header.Element("resultMsg"));
                        var body = xml.Element("body");
                        if (body == null)
                        {
                            Functions.MsgBox("서비스 오류 입니다.");
                            return;
                        }
                        PageNo = Functions.GetInt(body.Element("pageNo"));
                        TotalCount = Functions.GetInt(body.Element("totalCount"));

                        switch (ResultCode)
                        {
                            case "0":   //정상
                                var rows = from kkk in xml.Descendants("item")
                                           let mtime = Convert.ToInt32(Functions.GetString(kkk.Element("modifiedtime")).Substring(0, 8)).ToString("####-##-##")
                                           let fimage = Functions.GetString(kkk.Element("firstimage"))
                                           let fimage2 = Functions.GetString(kkk.Element("firstimage2"))
                                           select new ResultBaseModel
                                           {
                                               Addr1 = Functions.GetString(kkk.Element("addr1")),
                                               Addr2 = Functions.GetString(kkk.Element("addr2")),
                                               AreaCode = Functions.GetString(kkk.Element("areacode")),
                                               Cat1 = Functions.GetString(kkk.Element("cat1")),
                                               Cat2 = Functions.GetString(kkk.Element("cat2")),
                                               Cat3 = Functions.GetString(kkk.Element("cat3")),
                                               ContentId = Functions.GetString(kkk.Element("contentid")),
                                               ContentTypeId = Functions.GetString(kkk.Element("contenttypeid")),
                                               CreateTime = Functions.GetString(kkk.Element("createdtime")),
                                               FirstImage = fimage != null && fimage.Length > 0 ? new Uri(fimage) : null,
                                               FirstImage2 = fimage2 != null && fimage2.Length > 0 ? new Uri(fimage2) : null,
                                               ModifiedTime = mtime,
                                               ReadCount = Functions.GetString(kkk.Element("readcount")),
                                               SiGunGuCode = Functions.GetString(kkk.Element("sigungucode")),
                                               Title = Functions.GetString(kkk.Element("title")),
                                               MapX = Functions.GetString(kkk.Element("mapx")),
                                               MapY = Functions.GetString(kkk.Element("mapy")),
                                               MLevel = Functions.GetString(kkk.Element("mlevel")),
                                               BookTour = Functions.GetString(kkk.Element("booktour")),
                                               Tel = Functions.GetString(kkk.Element("tel")),
                                           };
                                ResultCollection = new System.Collections.ObjectModel.ObservableCollection<ResultBaseModel>(rows);
                                break;
                            case "22":  //서비스 요청제한횟수 초과에러
                                break;
                            default:
                                Functions.MsgBox("조회 중 오류가 발생했습니다.\n" + ResultMsg);
                                ResultCollection.Clear();
                                break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    Functions.MsgBox("오류" + ex.Message);
                }
            }

            IsBusy = false;
        }
    }
}
