﻿using KTour.Common;
using KTour.Interfaces;
using KTour.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace KTour.ViewModels
{
    public class SearchResultViewModel : KTour.Common.BindableBase
    {
        private string queryText;
        /// <summary>
        /// 검색 텍스트
        /// </summary>
        public string QueryText
        {
            get { return queryText; }
            set
            {
                queryText = value;
                OnPropertyChanged();
            }
        }

        private SearchFilterModel currentFilter;
        /// <summary>
        /// 현재 필터
        /// </summary>
        public SearchFilterModel CurrentFilter
        {
            get { return currentFilter; }
            set
            {
                currentFilter = value;
                OnPropertyChanged();
            }
        }

        private ObservableCollection<SearchFilterModel> searchFilterCollection;
        /// <summary>
        /// 검색 필터 목록
        /// </summary>
        public ObservableCollection<SearchFilterModel> SearchFilterCollection
        {
            get { return searchFilterCollection; }
            set
            {
                searchFilterCollection = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        /// 검색 결과
        /// </summary>
        public KTour.Common.CommonILC<IResultBaseModel> SearchResultCollection { get; set; }

        /// <summary>
        /// 키워드 검색 서비스 - 다음 데이터를 가지고오는 작업을 해야하기 때문에 계속 유지
        /// </summary>
        private KTour.OpenAPI.SearchKeywordService searchS;

        /// <summary>
        /// 생성자
        /// </summary>
        public SearchResultViewModel()
        {
            SearchFilterCollection = new ObservableCollection<SearchFilterModel>();
            SearchResultCollection = new Common.CommonILC<IResultBaseModel>();

            if (!Windows.ApplicationModel.DesignMode.DesignModeEnabled)
            {
                searchS = new OpenAPI.SearchKeywordService();

                SearchResultCollection.LoadMoreItemsEvent += 
                    (s, e) => 
                    {
                        //키워드가 존재할 때만..
                        if (searchS.Keyword != null && searchS.Keyword.Length > 0)
                        {
                            //스래드 처리
                            Functions.InvokeIfRequiredAsync(Functions.BaseContext,
                                _ =>
                                {
                                    searchProcess(null);
                                }, null);
                        }
                    };
            }
            else
            {
                var designDatas = Design.DesignData.ResultBaseCollection();
                foreach (var item in designDatas)
                {
                    SearchResultCollection.Add(item);
                }
                SearchFilterCollection.Add(new SearchFilterModel("테스트", 1000, true));
                SearchFilterCollection.Add(new SearchFilterModel("테스트2", 2000, false));
                QueryText = '\u201c' + "테스트" + '\u201d';
            }
        }

        /// <summary>
        /// 검색 조건 필더 추가
        /// </summary>
        /// <param name="queryText"></param>
        public void AddFilter(string queryText)
        {
            if (queryText != null && queryText != "")
            {
                if (SearchFilterCollection.Count(p => p.Query == queryText) == 0)
                {
                    //필터 추가
                    QueryText = '\u201c' + queryText + '\u201d';

                    var init = (from kkk in SearchFilterCollection
                                select kkk.Active = false).Count();

                    var filter = new SearchFilterModel(queryText, 0, true);
                    CurrentFilter = filter;
                    SearchFilterCollection.Add(filter);

                    //검색시작
                    searchProcess(CurrentFilter.Query);
                }
            }
        }

        /// <summary>
        /// 검색
        /// </summary>
        private async void searchProcess(string query)
        {
            //중복 실행 방지
            if (searchS.IsBusy == true) return;

            await searchS.Send(query);
            if (searchS.PageNo == 1) SearchResultCollection.Clear();
            foreach (var item in searchS.ResultCollection)
            {
                SearchResultCollection.Add(item);
            }
            //CommonILC 사용시 필수 지정 항목
            SearchResultCollection.HasMoreItems = searchS.TotalPage > searchS.PageNo ? true : false;
            SearchResultCollection.IsBusy = false;

            CurrentFilter.Count = searchS.TotalCount;
        }

        private DelegateCommand filterSelectCommand;
        /// <summary>
        /// 필더 선택 커맨드
        /// </summary>
        public DelegateCommand FilterSelectCommand
        {
            get 
            {
                if (filterSelectCommand == null)
                {
                    filterSelectCommand = new DelegateCommand(
                        args => 
                        {
                            var filter = args as SearchFilterModel;
                            if (filter != null)
                            {
                                var init = (from kkk in SearchFilterCollection
                                            select kkk.Active = false).Count();
                                
                                filter.Active = true;
                                CurrentFilter = filter;
                                searchProcess(CurrentFilter.Query);
                            }
                        });
                }
                return filterSelectCommand; 
            }
        }

        public void ItemClick(object item)
        {
            var fm = item as ResultBaseModel;
            if (fm != null)
            {
                //상세보기 페이지로 네비게이트
                Functions.NavigationPage(typeof(KTour.Views.DetailView), fm.ContentId + "|" + fm.ContentTypeId + "|" + fm.Title);
            }
        }

    }
}
